#ifndef IMREPOSITORYHANDLERIFACE_H_
#define IMREPOSITORYHANDLERIFACE_H_

#include "IMRepositoryHandler.h"

#ifdef __cplusplus
extern "C" {
#endif

/*
 * Creates a Directory in a given repositoPoints foundry
 * 
 * @param repository	where to put the new Directory
 * @param subDir		the subdirectory to be created.
 * 		if the repository is located in "/aaa", the current mount point is "bbb" and the subDir is "xyz"
 * 		the created directory will look like "/aaa/bbb/x1/x2/x3/xyz" where x1, x2 and x3 are hash
 * 		values generated from "xyz"
 * @param stat			int pointer that (if present) will store the resource status if it already exists
 * @param mustBeNew	bool indicating if the caller is certain that the resource does not exist yet 
 * 		(don't mark it as dirty if the repository is incomplete)
 * @return	0 if no error occured
 */
char* createDirectory( const char* repository, const char* subDir, int* stat = NULL, bool mustBeNew = false );

/*
 * Creates the path that will contain a file in a given repository
 * 
 * @param repository	where to put the new File
 * @param fileName	the file for which the path should be created.
 * 		if the repository is located in "/aaa", the current mount point is "bbb" and the file is "xyz"
 * 		the created directory will look like "/aaa/bbb/x1/x2/x3" where x1, x2 and x3 are hash 
 * 		values generated from "xyz"
 * @param stat			int pointer that (if present) will store the resource status if it already exists
 * @param mustBeNew	bool indicating if the caller is certain that the resource does not exist yet 
 * 		(don't mark it as dirty if the repository is incomplete)
 * @return	0 if no error occured
 */
char* createLocation( const char* repository, const char* fileName, int* stat = NULL, bool mustBeNew = false );

/*
 * Returns the path that contains a file/directory in a given repository
 * 
 * @param repository	where to locate the resource
 * @param resource	the file/directory that should be located.
 * @return	absolute path name of the requested resource, or empty string if not found
 */
char* getLocation( const char* repository, const char* resource, int* stat = NULL );

/*
 * this method checks the status of a given repository and if it does not match with the result of the previous 
 * check do certain tasks that the change justifies
 * @param repository	which repository to check
 * @return	int value describing the status of the repository (like REPOSITORY_STATUS_OK)
 */
int getRepositoryStatus( const char* repository );

/*
 * this method checks the status of a given resource in a given repository
 * @param repository	in which repository to check
 * @param resource	the file/directory that should be checked.
 * @return	int value describing the status of the resource (like RESOURCE_STATUS_OK)
 */
int checkResourceStatus( const char* repository, const char* resource );

/*
 * this method removes a resource (recursively if it is a directory) and hashed dirs that became empty
 * by this action
 * @param repository	from which repository to remove
 * @param resource	the file/directory that should be removed.
 * @return 0 if no error occured
 */
int removeResource( const char* repository, const char* resource );

/*
 * this method lists the files stored in a given repository 
 * @param repository	where to list the files
 * @param count				long* that will hold the size of the returned array
 * @param sort				indicates whether the result should be sorted
 * @return	string array with the filenames (without path, like "stuff.txt")
 */
char** listFiles( const char* repository, long* count, bool sort = false );

/*
 * this method lists the directories stored in a given repository 
 * @param repository	where to list the directories
 * @param count				long* that will hold the size of the returned array
 * @param sort				indicates whether the result should be sorted
 * @return	string array with the directorynames (like "xyz")
 */
char** listDirs( const char* repository, long* count, bool sort = false );

/*
 * this method checks the mounts, and if a mount is above move level, tries to move or delete 
 * files and directories until it goes below clean level
 * @param repository	where to check
 */
void checkOverload( const char* repository );

/*
 * this method tries to completely empty a mount in a repository by moving data safely to other mounts
 * @param repository	 	the repository
 * @param mountSubDir	the subdirectory in the repository that should be emptied
 */
int freeMount( const char* repository, const char* mountSubDir );

/**
 * If the resource is not on a 'fast' mount, try to find a 'fast' mount (whcich hasn't reached switch or full limit)
 *   and move the resource there
 *   access time will not change
 *
 * @param repository	 	the repository
 * @param	resource	the name of the resource to move
 * @return	the new location of the resource if it was in fact moved, NULL otherwise
 */
char * unAge( const char* repository, const char* resource );

#ifdef __cplusplus
}
#endif

#endif /*IMREPOSITORYHANDLERIFACE_H_*/
